//============================================================================//
//
//  VELOCITY DRIVERS
//
//    author:   Dave Rodgers  (a.k.a. trepan)
//    brief:    BZFS plugin to implement velocity drivers  (via physics drivers)
//    date:     June 28, 2010
//    license:  LGPL v2.1
//
//============================================================================//

#include <string>
#include <vector>
#include <set>
#include <map>
using std::string;

#include <stdio.h>

#include "bzfsAPI.h"

#include "phydrv.h"
#include "bzfs.h"
#include "StateDatabase.h"


//============================================================================//

BZ_GET_PLUGIN_VERSION


//============================================================================//

struct VelocityDriver {
  int id;
  string name;
  float linVel;
  float angVel;
};


typedef std::map<int, VelocityDriver> VeldrvMap;

static VeldrvMap velDrivers;

static bool triedWorld = false;


//============================================================================//

static bool getVelDrivers()
{
  if (!velDrivers.empty()) {
    return true;
  }

  if (triedWorld) {
    return true;
  }

  const PhydrvHandler::IdMap* idMap = PhydrvHandler::getIdMap();
  if (idMap == NULL) {
    return false;
  }

  triedWorld = true;

  PhydrvHandler::IdMap::const_iterator it;
  for (it = idMap->begin(); it != idMap->end(); ++it) {
    float lin, ang;
    const std::string& name = it->second;
    if (sscanf(name.c_str(), "veldrv_lin%f_ang%f", &lin, &ang) == 2) {
      VelocityDriver veldrv;
      veldrv.id = it->first;
      veldrv.name = name;
      veldrv.linVel = lin;
      veldrv.angVel = ang;
      velDrivers[veldrv.id] = veldrv;
      bz_debugMessagef(1,
        "Found veldrv: %i '%s' lin=%f ang=%f",
        veldrv.id, veldrv.name.c_str(),
        veldrv.linVel, veldrv.angVel
      );
    }
  }

  return true;
}


static const VelocityDriver* findVeldrv(int phydrv)
{
  if (!getVelDrivers()) {
    return NULL;
  }

  VeldrvMap::const_iterator it = velDrivers.find(phydrv);
  if (it == velDrivers.end()) {
    return NULL;
  }
  return &it->second;
}


//============================================================================//

static void sendPlayerBZDB(int playerID, const string& key,
                                         const string& value)
{
  void *bufStart = getDirectMessageBuffer();
  void *buf = nboPackUShort(bufStart, 1);
  buf = nboPackUByte (buf, key.length());
  buf = nboPackString(buf, key.c_str(), key.length());
  buf = nboPackUByte (buf, value.length());
  buf = nboPackString(buf, value.c_str(), value.length());

  const int len = (char*)buf - (char*)bufStart;
  directMessage(playerID, MsgSetVar, len, bufStart);
}


static void sendPlayerBZDB(int playerID, const string& key,
                                         const float value)
{
  char buf[64];
  snprintf(buf, sizeof(buf), "%f", value);
  sendPlayerBZDB(playerID, key, buf);
}


//============================================================================//

class PhydrvEvents : public PhydrvHandler
{
  void phydrvChange(int playerID, int newPhydrv, int oldPhydrv)
  {
    const VelocityDriver* newVeldrv = findVeldrv(newPhydrv);
    const VelocityDriver* oldVeldrv = findVeldrv(oldPhydrv);

    if (newVeldrv != oldVeldrv) {
      float linVel = BZDB.eval(StateDatabase::BZDB_TANKSPEED);
      float angVel = BZDB.eval(StateDatabase::BZDB_TANKANGVEL);
      if (newVeldrv != NULL) {
        linVel *= newVeldrv->linVel;
        angVel *= newVeldrv->angVel;
      }

      sendPlayerBZDB(playerID, "_tankSpeed",  linVel);
      sendPlayerBZDB(playerID, "_tankAngVel", angVel);
    }
  }
};


static PhydrvEvents phydrvEvents;


//============================================================================//

BZF_PLUGIN_CALL int bz_Load(const char* /*commandLine*/)
{
  phydrvEvents.insertEvents();
  bz_debugMessage(3, "veldrv plugin loaded");
  return 0;
}


//============================================================================//

BZF_PLUGIN_CALL int bz_Unload()
{
  phydrvEvents.removeEvents();
  bz_debugMessage(3, "veldrv plugin unloaded");
  return 0;
}


//============================================================================//


// Local Variables: ***
// mode:C++ ***
// tab-width: 8 ***
// c-basic-offset: 2 ***
// indent-tabs-mode: t ***
// End: ***
// ex: shiftwidth=2 tabstop=8

